

/*
 * Copyright (C) 2000,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * $Revision: 1.1.1.1 $
 * $Name: initial $
 * $Log: slam.c,v $
 * Revision 1.1.1.1  2000/01/05 21:49:36  wes
 * Initial checkin
 *
 *
 */
 
#include  "slam.h"

/*
 * command-line args:
 *
 * server side: slam -s[erver] 
 * client side: slam -c[lient] [-bs NN [set TCP window to NN bytes, default is 8192 bytes] [-h hostname [this argument is required on the client side, hostname of server (must be dns resolvable)] [-n XX (number of block transfers, default is 128)]
 *
 * only the client side needs to set the -bs argument
 */

static int myBlockSize=8192;
static char serverHostName[1024];
static int numTransfers=128;

typedef enum
{
    SERVER=0,
    CLIENT=1,
    UNKNOWN=-1
} servEnum;

static servEnum serverMode = UNKNOWN;

void
doServerCode(int blockSize)
{
    int listenfd, connfd;
    struct sockaddr_in cliaddr;
    int clilen;
    char buf[256];
    char *c;
/*    in_port_t port; */
    unsigned short port;
    int i;
    int sessionBlockSize, numTransfers;
    void *core;

    double accumSeconds=0.0;
    double accumKbytes=0;
    
    struct timeval t1, t2;
    double dt1,dt2,dt;

    
    listenfd = setupServerSocket(SLAM_SERVER_PORT, blockSize);
    memset(&cliaddr,0,sizeof(cliaddr));
    clilen = sizeof(cliaddr);

    /*
     *  the first connection that comes in will give us config
     *  data. it is handled differently.
     */

    connfd = accept(listenfd, (struct sockaddr *)&cliaddr, &clilen);
    
    c = inet_ntoa(cliaddr.sin_addr);
    port = ntohs(cliaddr.sin_port);

    /*
     * pre-transfer negotiation
     */
    readline(connfd,(void *)buf,256);
    sscanf(buf,"%*s %d", &sessionBlockSize);
    fprintf(stderr," server got a blocksize of %d. \n",sessionBlockSize);

    readline(connfd,(void *)buf,256);
    sscanf(buf,"%*s %d", &numTransfers);
    fprintf(stderr," number of transfers this session= %d. \n",numTransfers);

    core = (void *)malloc(sizeof(char)*sessionBlockSize);
    
    /*
     * begin block transfers & timing
     */

    for (i=0;i<numTransfers;i++)
    {
	double kBytesSec;
	
	gettimeofday(&t1,NULL);
	
	read_n(connfd, core, sessionBlockSize);
	
	gettimeofday(&t2,NULL);
	dt1 = t1.tv_sec + (double) t1.tv_usec/1000000.0;
	dt2 = t2.tv_sec + (double) t2.tv_usec/1000000.0;
	dt = dt2 - dt1;		/* in seconds */
	
	accumSeconds += dt;
	accumKbytes += (double)(sessionBlockSize)/(1024.);

	kBytesSec = accumKbytes/accumSeconds;

	fprintf(stderr," server receiving %lf kB/s, %lf kb/s \r", kBytesSec, kBytesSec * 8.0);
    }
    fprintf(stderr," \n");

    fprintf(stderr," %d bytes in %g real seconds = %g kB/sec\n",(int)(accumKbytes*1024.0), accumSeconds, accumKbytes/accumSeconds);

    free((void *)core);
}

void
doClientCode(int myBlockSize,
	     char *serverHostName,
	     int numTransfers)
{
    int outfd;
    char workbuf[256];
    int i;
    void *core;
    
    outfd = connectToServer(serverHostName, 1, myBlockSize);

    /* the first couple of transfers to the server set the blocksize, etc. */
    memset(workbuf,0,256);
    sprintf(workbuf,"BLOCKSIZE %d \n",myBlockSize);
    write_n(outfd, (void *)workbuf, strlen(workbuf));
    
    memset(workbuf,0,256);
    sprintf(workbuf,"NTRANSFERS %d \n",numTransfers);
    write_n(outfd, (void *)workbuf, strlen(workbuf));

    core = (void *)malloc(sizeof(char)*myBlockSize);
    memset((void *)core, 0xca, myBlockSize);
    
    for (i=0;i<numTransfers;i++)
    {
	write_n(outfd, core, myBlockSize);
    }

    close(outfd);
}

void
parseArgs(int ac,
	  char *av[])
{
    int i=1;
    ac--;

    while (ac > 0)
    {
	if (strcmp(av[i],"-bs") == 0)
	{
	    ac--;
	    i++;
	    sscanf(av[i],"%d",&myBlockSize);
	}
	else if (strcmp(av[i],"-n") == 0)
	{
	    ac--;
	    i++;
	    sscanf(av[i],"%d",&numTransfers);
	}
	else if (strncmp(av[i],"-h",2) == 0)
	{
	    ac--;
	    i++;
	    strcpy(serverHostName,av[i]);
	}
	else if (strcmp(av[i] ,"-s") == 0)
	{
	    serverMode = SERVER;
	}
	else if (strcmp(av[i],"-c") == 0)
	{
	    serverMode = CLIENT;
	}


	ac--;
	i++;
    }

    if (serverMode == UNKNOWN)
    {
	fprintf(stderr," error: must specify either server mode (-s) or client mode (-c) on the command line \n");
	exit(-1);
    }
}

main(int ac,
     char *av[])
{
    parseArgs(ac,av);

    if (serverMode == SERVER)
	doServerCode(myBlockSize);
    else
	doClientCode(myBlockSize,serverHostName,numTransfers);
}


